package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.bean.dto.ExamDownloadRecordDto;
import com.qmth.eds.common.entity.ExamDownloadRecord;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.enums.UseSceneEnum;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.mapper.ExamDownloadRecordMapper;
import com.qmth.eds.service.ExamDownloadRecordService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.StringJoiner;

@Service
public class ExamDownloadRecordServiceImpl extends ServiceImpl<ExamDownloadRecordMapper, ExamDownloadRecord> implements ExamDownloadRecordService {
    @Override
    public IPage<ExamDownloadRecordDto> list(Long semesterId, Long examTypeId, Long operateTime, Boolean used, String useScene, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        String operateDate = operateTime != null ? new SimpleDateFormat("yyyy-MM-dd").format(operateTime) : null;
        IPage<ExamDownloadRecordDto> examDownloadRecordDtoIPage = this.baseMapper.list(new Page<>(pageNumber, pageSize), schoolId, semesterId, examTypeId, operateDate, used, useScene);
        for (ExamDownloadRecordDto record : examDownloadRecordDtoIPage.getRecords()) {
            String scene = record.getUseScene();
            if (StringUtils.isNotBlank(scene)) {
                String[] strings = scene.split(",");
                StringJoiner stringJoiner = new StringJoiner(",");
                for (String string : strings) {
                    stringJoiner.add(UseSceneEnum.valueOf(string).getDesc());
                }
                record.setUseSceneStr(stringJoiner.toString());
            }
        }
        return examDownloadRecordDtoIPage;
    }

    @Transactional
    @Override
    public boolean used(String id, Boolean used, String useScene) {
        String[] ids = id.split(",");
        for (String s : ids) {
            ExamDownloadRecord downloadRecord = this.baseMapper.selectById(Long.parseLong(s));
            if (downloadRecord == null) {
                throw ExceptionResultEnum.ERROR.exception("下载记录数据不存在");
            }
            downloadRecord.setUsed(used);
            downloadRecord.setUseScene(useScene);
            this.baseMapper.updateById(downloadRecord);
        }
        return true;
    }
}
