package com.qmth.eds.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.googlecode.aviator.AviatorEvaluator;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.eds.bean.dto.ExamSyncStudentDto;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.*;
import com.qmth.eds.common.enums.*;
import com.qmth.eds.common.tools.CloudMarkingUtils;
import com.qmth.eds.common.tools.WhuUtils;
import com.qmth.eds.common.util.FileUtil;
import com.qmth.eds.service.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.RoundingMode;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 同步服务类
 */
@Service
public class WhuDataSyncServiceImpl implements WhuDataSyncService {

    private static final Logger log = LoggerFactory.getLogger(WhuDataSyncServiceImpl.class);

    @Resource
    private TBSyncTaskService tbSyncTaskService;

    @Resource
    private WhuUtils whuUtils;

    @Resource
    private CloudMarkingUtils cloudMarkingUtils;

    @Resource
    private CloudMarkingScoreService cloudMarkingScoreService;

    @Resource
    private CloudMarkingScoreForeignService cloudMarkingScoreForeignService;

    @Resource
    private ExamSyncStudentService examSyncStudentService;

    @Resource
    private ExamSyncStudentTempService examSyncStudentTempService;

    @Resource
    private ExamSyncTotalService examSyncTotalService;

    @Resource
    private BasicMessageService basicMessageService;

    @Resource
    private ExamAssignService examAssignService;

    @Async
    @Override
    public void syncWuhanUniversityExamData(ExamScheduleTask examScheduleTask, TBSyncTask tbSyncTask, boolean isAuto) {
        // 同步初始参数
        TaskResultEnum result = null;
        String errorMessage = "";
        Long schoolId = examScheduleTask.getSchoolId();
        String schoolName = examScheduleTask.getSchoolName();
        Long semesterId = examScheduleTask.getSemesterId();
        String semesterName = examScheduleTask.getSemesterName();
        Long examTypeId = examScheduleTask.getExamTypeId();
        String examTypeName = examScheduleTask.getExamTypeName();
        File txtFile = null;
        FileInputStream fis = null;
        // 汇总表ID
        Long examSyncTotalId = SystemConstant.getDbUuid();
        try {
            // 同步中
            tbSyncTaskService.updateStatusAndResultById(tbSyncTask.getId(), TaskStatusEnum.RUNNING, null, null);
            // 获取token
            String token = whuUtils.getAccessToken(schoolId);
            if (StringUtils.isBlank(token)) {
                throw ExceptionResultEnum.ERROR.exception("获取AccessToken失败");
            }
            // 调用接口
            List<ExamSyncStudentTemp> examSyncStudentTemps = whuUtils.getKwData(schoolId, token);

            examSyncStudentTemps.forEach(m -> {
                m.setId(SystemConstant.getDbUuid());
                m.setSchoolId(schoolId);
                m.setExamSyncTotalId(examSyncTotalId);
            });
            // 保存临时考务数据
            examSyncStudentTempService.saveBatch(examSyncStudentTemps);

            // 生成数据文件，用来比较内容是否有变动
            txtFile = createTxt(schoolId, examSyncStudentTemps);
            String txtFileMd5 = null;

            if (txtFile != null) {
                fis = new FileInputStream(txtFile);
                txtFileMd5 = DigestUtils.md5Hex(fis);
            }
            // 校验数据是否变动，若变动，需要新增
            if (canAddTotal(schoolId, semesterId, examTypeId, examSyncStudentTemps.size(), txtFileMd5)) {
                // 保存到正式表
                List<ExamSyncStudent> examSyncStudents = examSyncStudentTemps.stream().map(m -> {
                    ExamSyncStudent examSyncStudent = new ExamSyncStudent();
                    BeanUtil.copyProperties(m, examSyncStudent);
                    examSyncStudent.setId(SystemConstant.getDbUuid());
                    return examSyncStudent;
                }).collect(Collectors.toList());
                examSyncStudentService.saveBatch(examSyncStudents);

                // 将以前数据设置为不可下载
                examSyncTotalService.updateDownloadStatusBySchoolIdAndSemesterIdAndExamTypeId(schoolId, semesterId, examTypeId);

                // 生成汇总数据
                long syncDate;
                if (isAuto) {
                    ZoneId zone = ZoneId.systemDefault();
                    Instant instant = LocalDate.now().atStartOfDay().atZone(zone).toInstant();
                    Date date = Date.from(instant);
                    syncDate = date.getTime();
                } else {
                    syncDate = System.currentTimeMillis();
                }
                int colleges = Integer.parseInt(String.valueOf(examSyncStudentTemps.stream().map(ExamSyncStudentTemp::getJgmc).filter(StringUtils::isNotBlank).distinct().count()));
                int subjects = Integer.parseInt(String.valueOf(examSyncStudentTemps.stream().map(ExamSyncStudentTemp::getKch).filter(StringUtils::isNotBlank).distinct().count()));
                int students = examSyncStudentTemps.size();
                ExamSyncTotal examSyncTotal = new ExamSyncTotal(syncDate, schoolId, schoolName, semesterId, semesterName, examTypeId, examTypeName, colleges, subjects, students, txtFileMd5);
                examSyncTotal.setId(examSyncTotalId);
                examSyncTotalService.save(examSyncTotal);
                // 生成excel文件
                File excelFile = createExcel(schoolId, examSyncStudents);

                // 更新文件路径
                examSyncTotal.setFileName(excelFile.getName());
                examSyncTotal.setFilePath(excelFile.getPath());
                examSyncTotalService.updateById(examSyncTotal);

                // 下载过学校+学期+考试类型  文件的实施人员发送短信
                basicMessageService.sendTeachDataChangeNotice(schoolId, semesterId, examTypeId);
            }
            result = TaskResultEnum.SUCCESS;
        } catch (ApiException | IOException e) {
            result = TaskResultEnum.ERROR;
            errorMessage = errorMessage + e.getMessage();
        } finally {
            // 删除临时表数据
            examSyncStudentTempService.deleteByExamSyncTotalId(examSyncTotalId);
            // 更新任务状态
            tbSyncTaskService.updateStatusAndResultById(tbSyncTask.getId(), TaskStatusEnum.FINISH, result, errorMessage);

            // 关闭fis流
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            // 删除txt文件
            if (txtFile != null) {
                txtFile.delete();
            }
        }
    }

    @Override
    public void syncCloudMarkingStudentScoreData(Long schoolId, TBSyncTask tbSyncTask, Integer examId) {
        // 同步初始参数
        TaskResultEnum result = null;
        String errorMessage = "";
        Long collegeId = tbSyncTask.getSchoolId();
        Long semesterId = tbSyncTask.getSemesterId();
        Long examTypeId = tbSyncTask.getExamTypeId();
        File txtFile = null;
        FileInputStream fis = null;
        // 汇总表ID
        try {
            // 同步中
            tbSyncTaskService.updateStatusAndResultById(tbSyncTask.getId(), TaskStatusEnum.RUNNING, null, null);

            Map<String, String> mapSource = new HashMap<>();
            mapSource.put("examId", String.valueOf(examId));
            int totalCount = cloudMarkingUtils.getStudentCount(schoolId, mapSource, collegeId);
            List<CloudMarkingScore> cloudMarkingScoreList = new ArrayList<>();
            if (totalCount > 0) {
                int pageSize = SystemConstant.PAGE_SIZE;
                int mod = totalCount % pageSize;
                int pageNos = mod == 0 ? totalCount / pageSize : totalCount / pageSize + 1;
                for (int i = 1; i <= pageNos; i++) {
                    List<JSONObject> students = cloudMarkingUtils.getStudentScore(schoolId, mapSource, collegeId, i, pageSize);
                    List<CloudMarkingScore> cloudMarkingScores = students.stream().map(m -> {
                        CloudMarkingScore cloudMarkingScore = new CloudMarkingScore();
                        cloudMarkingScore.setId(SystemConstant.getDbUuid());
                        cloudMarkingScore.setSchoolId(collegeId);
                        cloudMarkingScore.setSemesterId(semesterId);
                        cloudMarkingScore.setExamTypeId(examTypeId);
                        cloudMarkingScore.setExamId(examId);
                        cloudMarkingScore.setExamCode(m.getString("examCode"));
                        cloudMarkingScore.setExamNumber(m.getString("examNumber"));
                        cloudMarkingScore.setStudentCode(m.getString("studentCode"));
                        cloudMarkingScore.setName(m.getString("name"));
                        cloudMarkingScore.setSubjectCode(m.getString("subjectCode"));
                        cloudMarkingScore.setSubjectName(m.getString("subjectName"));
                        cloudMarkingScore.setPaperType(m.getString("paperType"));
                        cloudMarkingScore.setCollege(m.getString("college"));
                        cloudMarkingScore.setClassName(m.getString("className"));
                        cloudMarkingScore.setTeacher(m.getString("teacher"));
                        cloudMarkingScore.setStatus(m.getInteger("status"));
//                        cloudMarkingScore.setSheetUrls(m.getString("sheetUrls"));
                        cloudMarkingScore.setTotalScore(m.getString("totalScore"));
                        cloudMarkingScore.setObjectiveScore(m.getString("objectiveScore"));
                        cloudMarkingScore.setSubjectiveScore(m.getString("subjectiveScore"));
                        cloudMarkingScore.setObjectiveScoreDetail(m.getString("objectiveScoreDetail"));
                        cloudMarkingScore.setSubjectiveScoreDetail(m.getString("subjectiveScoreDetail"));
//                        cloudMarkingScore.setMarkTags(m.getString("markTags"));
                        return cloudMarkingScore;
                    }).collect(Collectors.toList());
                    cloudMarkingScoreList.addAll(cloudMarkingScores);
                }
            }

            //2.调用教务系统接口回传成绩
            if (!cloudMarkingScoreList.isEmpty()) {
                saveScoreData(schoolId, collegeId, semesterId, examTypeId, examId, cloudMarkingScoreList);
            }

            result = TaskResultEnum.SUCCESS;
        } catch (ApiException e) {
            result = TaskResultEnum.ERROR;
            errorMessage = errorMessage + e.getMessage();
        } finally {
            // 删除临时表数据
            // 更新任务状态
            tbSyncTaskService.updateStatusAndResultById(tbSyncTask.getId(), TaskStatusEnum.FINISH, result, errorMessage);

            // 关闭fis流
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            // 删除txt文件
            if (txtFile != null) {
                txtFile.delete();
            }
        }
    }

    @Override
    public void calcAssignScore(ExamAssign examAssign) {
        examAssign.setStatus(ExamAssignStatusEnum.RUNNING);
        examAssignService.updateById(examAssign);
        String errorMsg = null;
        try {
            // 选模式
            // 解析计算参数
            JSONObject object = JSON.parseObject(examAssign.getCoefficient());
            Boolean all = object.getBoolean("all");
            Double allValue = object.getDoubleValue("value");

            List<ExamSyncStudentDto> finalList = new ArrayList<>();
            List<ExamSyncStudentDto> examSyncStudentDtoList = cloudMarkingScoreService.listExamSyncStudentDtos(examAssign.getSchoolId(), examAssign.getSemesterId(), examAssign.getExamTypeId(), examAssign.getCourseCode(), examAssign.getOpenCollege());
            // 初始卷面最高分
            Double maxTotalScore = examSyncStudentDtoList.stream().map(m -> Double.parseDouble(m.getTotalScore())).max(Comparator.comparingDouble(x -> x)).get();
            if (all) {
                // 卷面最高分
                calcProgress(examAssign.getFormula(), allValue, examSyncStudentDtoList, maxTotalScore, finalList);
            } else {
                Map<String, List<ExamSyncStudentDto>> collect = examSyncStudentDtoList.stream().collect(Collectors.groupingBy(ExamSyncStudentDto::getJgmc));
                for (Map.Entry<String, List<ExamSyncStudentDto>> entry : collect.entrySet()) {
                    String college = entry.getKey();
                    List<JSONObject> details = JSON.parseArray(object.getString("detail"), JSONObject.class);
                    Optional<JSONObject> objectOptional = details.stream().filter(m -> m.getString("college").equals(college)).findFirst();
                    if (!objectOptional.isPresent()) {
                        throw ExceptionResultEnum.ERROR.exception(String.format("开课学院[%s]未设置的参数值", college));
                    }
                    double collegeValue = objectOptional.get().getDoubleValue("value");
                    calcProgress(examAssign.getFormula(), collegeValue, entry.getValue(), maxTotalScore, finalList);
                }
            }

            // 更新云阅卷同步表中的赋分值
            if (CollectionUtils.isEmpty(finalList)) {
                throw ExceptionResultEnum.ERROR.exception("赋分值计算失败，请联系管理员");
            }

            for (ExamSyncStudentDto studentDto : finalList) {
                UpdateWrapper<CloudMarkingScore> updateWrapper = new UpdateWrapper<>();
                updateWrapper.lambda().set(CloudMarkingScore::getAssignScore, studentDto.getAssignScore())
                        .eq(CloudMarkingScore::getId, studentDto.getCloudMarkingScoreId());
                cloudMarkingScoreService.update(updateWrapper);
            }

        } catch (Exception e) {
            errorMsg = e.getMessage();
        } finally {
            examAssign.setStatus(ExamAssignStatusEnum.FINISH);
            examAssign.setErrorMsg(errorMsg);
            // 重新计算后，发布状态改为未发布
            examAssign.setPublish(false);
            examAssignService.updateById(examAssign);
        }
    }

    /**
     * 计算公共方法
     *
     * @param formula                公式
     * @param value                  系数
     * @param examSyncStudentDtoList 源数据
     * @param finalList              计算后数据
     *                               备注：AviatorEvaluator.compile(formula).execute(paramMap),paramMap中的参数值不能为String类型
     */
    private void calcProgress(FormulaEnum formula, Double value, List<ExamSyncStudentDto> examSyncStudentDtoList, Double maxTotalScore, List<ExamSyncStudentDto> finalList) {
        // 公式1（赋值分=卷面成绩+ (100- 卷面成绩)/赋分系数）
        if (FormulaEnum.FORMULA1.equals(formula)) {
            // a为卷面成绩，y为赋分系数
            String formula1 = "a+(100-a)/b";
            for (ExamSyncStudentDto studentDto : examSyncStudentDtoList) {
                Map<String, Object> paramMap = new HashMap<>();
                paramMap.put("a", Double.parseDouble(studentDto.getTotalScore()));
                paramMap.put("b", value);
                String assignScore = AviatorEvaluator.compile(formula1).execute(paramMap).toString();
                studentDto.setAssignScore(stripTrailingZerosFormula1(assignScore));
                finalList.add(studentDto);
            }
        }
        // 公式2
        /*
         * 赋值分=ROUND(IF(初始卷面>=参数,55+(初始卷面-参数)/（初始卷面最高分-参数)*（x-55）,初始卷面*50/参数)*2,0)/2
            说明：1.如果初始卷面最高的大于99时，则x=100，否则x=99；
                 2.参数根据每次总体初始成绩的不及格率进行调整。
         */
        else if (FormulaEnum.FORMULA2.equals(formula)) {
            int x = maxTotalScore.doubleValue() > 99 ? 100 : 99;
            // a:初始卷面，b:参数，c:初始卷面最高分，x:说明1中参数
            String formula2_1 = "(55+(a-b)/(c-b)*(x-55))*2";
            String formula2_2 = "(a*50/b)*2";

            for (ExamSyncStudentDto studentDto : examSyncStudentDtoList) {
                Map<String, Object> paramMap = new HashMap<>();
                Double totalScore = Double.valueOf(studentDto.getTotalScore());
                String tempAssignScore;
                if (totalScore >= value) {
                    paramMap.put("a", totalScore.doubleValue());
                    paramMap.put("b", value);
                    paramMap.put("c", maxTotalScore.doubleValue());
                    paramMap.put("x", x);
                    tempAssignScore = AviatorEvaluator.compile(formula2_1).execute(paramMap).toString();
                } else {
                    paramMap.put("a", totalScore.doubleValue());
                    paramMap.put("b", value.doubleValue());
                    tempAssignScore = AviatorEvaluator.compile(formula2_2).execute(paramMap).toString();
                }

                BigDecimal bigDecimal = new BigDecimal(tempAssignScore).setScale(0, RoundingMode.HALF_UP);
                String assignScore = bigDecimal.divide(new BigDecimal("2")).toString();
                studentDto.setAssignScore(stripTrailingZeros(assignScore));
                finalList.add(studentDto);
            }
        }
    }

    /**
     * 处理公式1计算出来的赋分值
     *
     * @param value
     * @return
     */
    private String stripTrailingZerosFormula1(String value) {
        BigDecimal valueDecimal = new BigDecimal(value);
        // 处理分数 86.221->86,86.34->86.5,86.664->87
        int index = value.indexOf(".");
        if (index > 0) {
            // 小数部分值
            double decimalPart = new BigDecimal("0" + value.substring(index)).doubleValue();
            if (decimalPart < 0.5) {
                String divide = valueDecimal.multiply(new BigDecimal("2")).setScale(0, RoundingMode.HALF_UP).divide(new BigDecimal("2")).toString();
                value = stripTrailingZeros(divide);
            } else {
                value = valueDecimal.setScale(0, RoundingMode.HALF_UP).toString();
            }
        }
        return stripTrailingZeros(value);
    }

    /**
     * 处理公式2计算出来的赋分值
     *
     * @param value
     * @return
     */
    private String stripTrailingZeros(String value) {
        return new BigDecimal(value).stripTrailingZeros().toPlainString();
    }

    @Transactional
    public void saveScoreData(Long schoolId, Long collegeId, Long semesterId, Long examTypeId, Integer examId, List<CloudMarkingScore> cloudMarkingScoreList) {
        // 查询对外数据
        ExamSyncTotal examSyncTotal = examSyncTotalService.getBySemesterIdAndExamTypeIdAndUseFile(schoolId, semesterId, examTypeId, true);
        if (examSyncTotal == null) {
            throw ExceptionResultEnum.ERROR.exception("未设置使用数据文件，请联系管理员处理");
        }
        List<ExamSyncStudent> examSyncStudents = examSyncStudentService.listByExamSyncTotalId(examSyncTotal);
        for (CloudMarkingScore cloudMarkingScore : cloudMarkingScoreList) {
            Optional<ExamSyncStudent> optional = examSyncStudents.stream()
                    .filter(s -> s.getJxbmc().equals(cloudMarkingScore.getClassName())
                            && ((StringUtils.isBlank(s.getCloudMarkingCourseCode()) && s.getKch().equals(cloudMarkingScore.getSubjectCode())) || (StringUtils.isNotBlank(s.getCloudMarkingCourseCode()) && s.getCloudMarkingCourseCode().equals(cloudMarkingScore.getSubjectCode())))
                            && s.getXh().equals(cloudMarkingScore.getStudentCode())).findFirst();
            if (optional.isPresent()) {
                ExamSyncStudent examSyncStudent = optional.get();
                // 匹配上，保存考务数据表ID
                cloudMarkingScore.setExamSyncStudentId(examSyncStudent.getId());
            }
        }

        // 删除旧数据
        cloudMarkingScoreService.deleteByKeys(collegeId, semesterId, examTypeId, examId);
        List<CloudMarkingScore> data = new ArrayList<>();
        //1000条提交一次
        for (CloudMarkingScore markingScore : cloudMarkingScoreList) {
            if (data.size() == 1000) {
                cloudMarkingScoreService.saveBatch(data);
                data.clear();
            }
            data.add(markingScore);
        }
        //将剩下的数据也导入
        if (!data.isEmpty()) {
            cloudMarkingScoreService.saveBatch(data);
        }

        /*// 查询对外数据
        ExamSyncTotal examSyncTotal = examSyncTotalService.getBySemesterIdAndExamTypeIdAndUseFile(schoolId, semesterId, examTypeId, true);
        if (examSyncTotal == null) {
            throw ExceptionResultEnum.ERROR.exception("未设置使用数据文件，请联系管理员处理");
        }
        List<CloudMarkingScoreForeign> cloudMarkingScoreForeigns = new ArrayList<>();
        List<ExamSyncStudent> examSyncStudents = examSyncStudentService.listByExamSyncTotalId(examSyncTotal);
        for (CloudMarkingScore cloudMarkingScore : cloudMarkingScoreList) {
            Optional<ExamSyncStudent> optional = examSyncStudents.stream()
                    .filter(s -> s.getJxbmc().equals(cloudMarkingScore.getClassName())
                            && ((StringUtils.isBlank(s.getCloudMarkingCourseCode()) && s.getKch().equals(cloudMarkingScore.getSubjectCode())) || (StringUtils.isNotBlank(s.getCloudMarkingCourseCode()) && s.getCloudMarkingCourseCode().equals(cloudMarkingScore.getSubjectCode())))
                            && s.getXh().equals(cloudMarkingScore.getStudentCode())).findFirst();
            if (!optional.isPresent()) {
                cloudMarkingScoreForeigns.add(new CloudMarkingScoreForeign(collegeId, semesterId, examTypeId, examId, null, null, null, cloudMarkingScore.getSubjectCode(), cloudMarkingScore.getSubjectName(), cloudMarkingScore.getStudentCode(), cloudMarkingScore.getTotalScore()));
            } else {
                ExamSyncStudent examSyncStudent = optional.get();
                // 匹配上，保存考务数据表ID
                cloudMarkingScore.setExamSyncStudentId(examSyncStudent.getId());
                cloudMarkingScoreForeigns.add(new CloudMarkingScoreForeign(collegeId, semesterId, examTypeId, examId, examSyncStudent.getXnm(), examSyncStudent.getXqm(), examSyncStudent.getJxbId(), examSyncStudent.getKch(), examSyncStudent.getKcmc(), examSyncStudent.getXh(), cloudMarkingScore.getTotalScore()));
            }
        }

        // 删除旧数据
        cloudMarkingScoreService.deleteByKeys(collegeId, semesterId, examTypeId, examId);
        List<CloudMarkingScore> data = new ArrayList<>();
        //1000条提交一次
        for (CloudMarkingScore markingScore : cloudMarkingScoreList) {
            if (data.size() == 1000) {
                cloudMarkingScoreService.saveBatch(data);
                data.clear();
            }
            data.add(markingScore);
        }
        //将剩下的数据也导入
        if (!data.isEmpty()) {
            cloudMarkingScoreService.saveBatch(data);
        }

        // 删除全量数据
        UpdateWrapper<CloudMarkingScoreForeign> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(CloudMarkingScoreForeign::getSchoolId, collegeId)
                .eq(CloudMarkingScoreForeign::getSemesterId, semesterId)
                .eq(CloudMarkingScoreForeign::getExamTypeId, examTypeId)
                .eq(CloudMarkingScoreForeign::getExamId, examId);
        cloudMarkingScoreForeignService.remove(updateWrapper);

        List<CloudMarkingScoreForeign> foreignData = new ArrayList<>();
        //1000条提交一次
        for (CloudMarkingScoreForeign scoreForeign : cloudMarkingScoreForeigns) {
            if (foreignData.size() == 1000) {
                cloudMarkingScoreForeignService.saveBatch(foreignData);
                foreignData.clear();
            }
            foreignData.add(scoreForeign);
        }
        //将剩下的数据也导入
        if (!foreignData.isEmpty()) {
            cloudMarkingScoreForeignService.saveBatch(foreignData);
        }*/
    }

    /**
     * 检查数据是否有变动，需要新增一条新同步记录
     *
     * @param schoolId   学校ID
     * @param semesterId 学期ID
     * @param examTypeId 考试类型ID
     * @param count      考生数量
     * @param txtFileMd5 txt文件md5
     * @return true:可以新增  false:不能新增
     */
    private boolean canAddTotal(Long schoolId, Long semesterId, Long examTypeId, int count, String txtFileMd5) {
        // txtFileMd5为空，无法校验，返回true
        if (txtFileMd5 == null) {
            return true;
        }

        // 查询上一次同步数据，先比较数量
        QueryWrapper<ExamSyncTotal> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamSyncTotal::getSchoolId, schoolId)
                .eq(ExamSyncTotal::getSemesterId, semesterId)
                .eq(ExamSyncTotal::getExamTypeId, examTypeId)
                .orderByDesc(ExamSyncTotal::getSyncDate)
                .orderByDesc(ExamSyncTotal::getCreateTime);
        List<ExamSyncTotal> examSyncTotals = examSyncTotalService.list(queryWrapper);
        // 没有同步记录，返回true
        if (examSyncTotals.isEmpty()) {
            return true;
        }
        ExamSyncTotal examSyncTotal = examSyncTotals.get(0);
        // 上一次同步数据量与本次不一样，返回true
        if (count != examSyncTotal.getStudents()) {
            return true;
        }
        // 数据内容有变动，返回 true
        return !txtFileMd5.equals(examSyncTotal.getDataMd5());
    }

    /**
     * 生成txt，比较md5
     *
     * @param examSyncStudentTemps list
     */
    private File createTxt(Long schoolId, List<ExamSyncStudentTemp> examSyncStudentTemps) {
        // 按学号排序
        examSyncStudentTemps.sort(Comparator.comparing(ExamSyncStudentTemp::getXh));

        List<String> list = examSyncStudentTemps.stream().map(m -> {
            StringJoiner stringJoiner = new StringJoiner("|");
            stringJoiner.add(m.getXnm())
                    .add(m.getXqm())
                    .add(m.getJxbId())
                    .add(m.getJxbmc())
                    .add(m.getKch())
                    .add(m.getKcmc())
                    .add(m.getJgh())
                    .add(m.getXm())
                    .add(m.getKkbm())
                    .add(m.getXf())
                    .add(m.getXh())
                    .add(m.getXsxm())
                    .add(m.getJgmc())
                    .add(m.getZymc())
                    .add(m.getCxbj())
                    .add(m.getNjdmId())
                    .add(m.getZwh())
                    .add(m.getCdmc())
                    .add(m.getKsbz());
            return stringJoiner.toString();
        }).collect(Collectors.toList());

        File file = null;
        try {
            String fileName = "file-folder" + File.separator + schoolId + File.separator + System.currentTimeMillis() + ".txt";
            file = FileUtil.writeTxt(fileName, list);
        } catch (IOException e) {
            log.error("数据txt文件生成失败");
        }
        return file;
    }

    /**
     * 生成excel文件
     *
     * @param examSyncStudents 考务数据
     */
    private File createExcel(Long schoolId, List<ExamSyncStudent> examSyncStudents) throws IOException {
        List<List<String>> head = new ArrayList<>();
        List<String> headTitle0 = new ArrayList<>();
        headTitle0.add("学年");
        List<String> headTitle1 = new ArrayList<>();
        headTitle1.add("学期");
        List<String> headTitle2 = new ArrayList<>();
        headTitle2.add("教学班ID");
        List<String> headTitle3 = new ArrayList<>();
        headTitle3.add("教学班名称");
        List<String> headTitle4 = new ArrayList<>();
        headTitle4.add("课程号");
        List<String> headTitle5 = new ArrayList<>();
        headTitle5.add("课程名称");
        List<String> headTitle6 = new ArrayList<>();
        headTitle6.add("工号");
        List<String> headTitle7 = new ArrayList<>();
        headTitle7.add("姓名");
        List<String> headTitle8 = new ArrayList<>();
        headTitle8.add("开课部门");
        List<String> headTitle9 = new ArrayList<>();
        headTitle9.add("学分");
        List<String> headTitle10 = new ArrayList<>();
        headTitle10.add("学号");
        List<String> headTitle11 = new ArrayList<>();
        headTitle11.add("学生姓名");
        List<String> headTitle12 = new ArrayList<>();
        headTitle12.add("学院");
        List<String> headTitle13 = new ArrayList<>();
        headTitle13.add("专业");
        List<String> headTitle14 = new ArrayList<>();
        headTitle14.add("重修标记");
        List<String> headTitle15 = new ArrayList<>();
        headTitle15.add("年级");
        List<String> headTitle16 = new ArrayList<>();
        headTitle16.add("座位号");
        List<String> headTitle17 = new ArrayList<>();
        headTitle17.add("场地名称");
        List<String> headTitle18 = new ArrayList<>();
        headTitle18.add("考试备注");


        head.add(headTitle0);
        head.add(headTitle1);
        head.add(headTitle2);
        head.add(headTitle3);
        head.add(headTitle4);
        head.add(headTitle5);
        head.add(headTitle6);
        head.add(headTitle7);
        head.add(headTitle8);
        head.add(headTitle9);
        head.add(headTitle10);
        head.add(headTitle11);
        head.add(headTitle12);
        head.add(headTitle13);
        head.add(headTitle14);
        head.add(headTitle15);
        head.add(headTitle16);
        head.add(headTitle17);
        head.add(headTitle18);

        String fileName = "file-folder" + File.separator + schoolId + File.separator + System.currentTimeMillis() + ".xlsx";
        File file = new File(fileName);
        if (!file.exists()) {
            file.createNewFile();
        }
        ExcelWriter excelWriter = EasyExcel.write(file).build();
        //动态表头
        WriteSheet writeSheet1 = EasyExcel.writerSheet(0, "考务数据").head(head).build();
        excelWriter.write(examSyncStudents, writeSheet1);
        excelWriter.finish();

        return file;
    }
}
